package question16

type Vertex struct {
	Succ, Pred []int
}

type Graph struct {
	Order    int
	Vertices []Vertex
}

/*
 * The type SpanTree store spanning tree and their extra edges.
 * It provides the following operations
 * func (st *SpanTree) AddTreeEdge(x, y int)
 * st.AddTreeEdge(x,y) add the edge (x,y) to the tree
 * func (st *SpanTree) AddBackEdge(x, y int)
 * st.AddBackEdge(x, y) add the back edge (x,y) to the tree
 * func (st *SpanTree) AddForwardEdge(x, y int)
 * st.AddForwardEdge(x, y) add the forward edge (x,y) to the tree
 * func (st *SpanTree) AddCrossEdge(x, y int)
 * st.AddCrossEdge(x, y) add the cross edge (x,y) to the tree
 */

func (g *Graph) dfs(v int, pre, post []int, count *int, mark []bool, st *SpanTree) {
	*count += 1
	pre[v] = *count
	for _, s := range g.Vertices[v].Succ {
		if !mark[s] {
			mark[s] = true
			st.AddTreeEdge(v, s)
			g.dfs(s, pre, post, count, mark, st)
		} else {
			if post[s] == 0 {
				st.AddBackEdge(v, s)
			} else {
				if pre[v] < pre[s] {
					st.AddForwardEdge(v, s)
				} else {
					st.AddCrossEdge(v, s)
				}
			}
		}
	}
	*count += 1
	post[v] = *count
}

func (g *Graph) DFSBuildSpanningTree(v int, st *SpanTree) {
	pre := make([]int, g.Order)
	post := make([]int, g.Order)
	mark := make([]bool, g.Order)
	for i := range pre {
		pre[i] = 0
		post[i] = 0
		mark[i] = false
	}
	count := 0
	mark[v] = true
	g.dfs(v, pre, post, &count, mark, st)
	for v := range g.Vertices {
		if !mark[v] {
			mark[v] = true
			g.dfs(v, pre, post, &count, mark, st)
		}
	}
}
