package main

import (
	"flag"
	"fmt"
	"os"
)

type Vertex struct {
	Succ, Pred []int
}

type Graph struct {
	Order    int
	Vertices []Vertex
}

func (g *Graph) isLinked(x, y int) bool {
	for _, s := range g.Vertices[x].Succ {
		if s == y {
			return true
		}
	}
	return false
}

func (g *Graph) AddEdge(x, y int) {
	if x < 0 || x >= g.Order || y < 0 || y >= g.Order {
		panic("Vertices not in graph")
	}
	if !g.isLinked(x, y) {
		g.Vertices[x].Succ = append(g.Vertices[x].Succ, y)
		g.Vertices[y].Pred = append(g.Vertices[y].Pred, x)
	}
}

func NewGraph(order int) *Graph {
	return &Graph{order, make([]Vertex, order)}
}

func ReadGraph(fname string) (g *Graph) {
	file, err := os.Open(fname)
	if err != nil {
		panic(err)
	}
	defer file.Close()
	order := 0
	c, err := fmt.Fscanln(file, &order)
	if c < 1 || err != nil {
		panic(err)
	}
	g = NewGraph(order)
	for {
		var x, y int
		if i, err := fmt.Fscanln(file, &x, &y); err != nil || i == 0 {
			break
		} else {
			g.AddEdge(x, y)
		}
	}
	return g
}

func (g *Graph) ToDot(dot_name string) {
	file, err := os.Create(dot_name)
	if err != nil {
		panic(err)
	}
	defer file.Close()
	fmt.Fprintln(file, "digraph G {")
	for v, vert := range g.Vertices {
		for _, s := range vert.Succ {
			fmt.Fprintf(file, "  %v -> %v;\n", v, s)
		}
	}
	fmt.Fprintln(file, "}")
}

func main() {
	graph_name := "graph.nde"
	dot_name := "graph.dot"
	flag.StringVar(&graph_name, "graph", "graph.nde", "graph file name")
	flag.StringVar(&dot_name, "dot", "graph.dot", "dot file name")
	flag.Parse()
	g := ReadGraph(graph_name)
	g.ToDot(dot_name)
}
